import $ from 'jquery'
import 'jquery-contextmenu'
import copy from 'copy-text-to-clipboard'

function renderTree(tree)
{
    if (!tree) return []

    let recRenderTree = (tree, indent) => {
        return _.flatMapDeep(tree, (item) => [
            _.repeat("    ", item.indented-indent) + item.text,
            recRenderTree(item.children, indent),
        ])
    }

    return recRenderTree(tree, tree[0].indented)
}

function connectContextMenu(editor) {
    $.contextMenu({
        selector: '.marker',
        items: {
            createNewPage: {
                name: 'Create page from item',
                callback: function (key, opt) {
                    console.log('Create page from item', key, opt)
                },
                className: 'action-new-page'
            },
            copy: {
                name: 'Copy',
                callback: function (key, opt) {
                    editor.copy(this, {recursive: true}).then(result => {
                        copy(renderTree(result).join("\n"))
                    })
                },
                className: 'action-copy'
            },
            copyLine: {
                name: 'Copy line',
                callback: function (key, opt) {
                    editor.copy(this, {recursive: false}).then(result => {
                        copy(result.text)
                    })
                },
                className: 'action-copy-line'
            }
        }
    });
}

export default {
    connectContextMenu: connectContextMenu
}
